#ifndef _MESH_H_
#define _MESH_H_

#include "cglib/vector3.h"
#include "cglib/point3.h"

//! An interface represents a polygonal mesh.
class Mesh
{
public:
    //! An empty constructor.
    Mesh();
    
    //! An empty destructor.
    virtual ~Mesh();
    
    //! Return the number of vertex positions.
    virtual int get_position_count() const = 0;
    
    //! Return the vertex position with the given index.
    virtual Point3 get_position(int position_index) const = 0;
    
    //! Return the number of vertex normals.
    virtual int get_normal_count() const = 0;
    
    //! Return the vertex normal with the given index.
    virtual Vector3 get_normal(int normal_index) const = 0;
    
    //! Return the number of faces.
    virtual int get_face_count() const = 0;
    
    //! Return the number of vertices in the face with the given index.
    virtual int get_face_vertex_count(int face_index) const = 0;
    
    //! Return the index of the position of the vertex specified by \a vertex_index of the face specfied by \a face_index
    virtual int get_face_vertex_position_index(int face_index, int vertex_index) const = 0;
    
    //! Return the index of the normal of the vertex specified by \a vertex_index of the face specfied by \a face_index
    virtual int get_face_vertex_normal_index(int face_index, int vertex_index) const = 0;
    
    //! Return position of the vertex specified by \a vertex_index of the face specfied by \a face_index
    virtual Point3 get_face_vertex_position(int face_index, int vertex_index) const = 0;
    
    //! Return normal of the vertex specified by \a vertex_index of the face specfied by \a face_index
    virtual Vector3 get_face_vertex_normal(int face_index, int vertex_index) const = 0;
};

#endif /* _MESH_H_ */
