#ifndef _UNIVERSAL_MESH_H_
#define _UNIVERSAL_MESH_H_

#include "cglib/mesh.h"

//! An implementation of Mesh where the user is allowed to add positions, normals, and faces.
class UniversalMesh : public Mesh
{
public:
    //! An empty constructor.
    UniversalMesh();
    
    //! An empty destructor.
    virtual ~UniversalMesh();

    virtual int get_position_count() const;
    virtual Point3 get_position(int position_index) const;  
    virtual int get_normal_count() const;
    virtual Vector3 get_normal(int normal_index) const;    
    virtual int get_face_count() const;
    virtual int get_face_vertex_count(int face_index) const;
    virtual int get_face_vertex_position_index(int face_index, int vertex_index) const;
    virtual int get_face_vertex_normal_index(int face_index, int vertex_index) const;
    virtual Point3 get_face_vertex_position(int face_index, int vertex_index) const;
    virtual Vector3 get_face_vertex_normal(int face_index, int vertex_index) const;

    //! Add a new position. The newly added position will have the largest index.
    virtual void append_position(const Point3 &position);
    
    //! Add a new normal. The newly added normal will ahve the largest index.
    virtual void append_normal(const Vector3 &normal);
    
    //! Create a new face and add it to the collection of faces. The newly added face will have the largest index.
    virtual void append_new_face();
    
    //! Add a vertex, by specifying the indices of its position and normal, to the last face added via \c append_new_face().
    virtual void append_vertex_to_last_face(int position_index, int normal_index);
        
private:
    struct UniversalMeshImplementation;
    UniversalMeshImplementation *self;
};

#endif /* _UNIVERSAL_MESH_CPP_ */
